@@ -1,8 +1,4 @@
-"""
-xarray-einstats.
-
-Stats, linear algebra and einops for xarray
-"""
+"""Stats, linear algebra and einops for xarray."""
 
 from __future__ import annotations
 
@@ -10,4 +6,4 @@ from .linalg import einsum, raw_einsum, einsum_path, matmul
 
 __all__ = ["einsum", "raw_einsum", "einsum_path", "matmul"]
 
-__version__ = "0.3.0"
+__version__ = "0.4.0"
